import pyperclip

def copy_to_clipboard(data: str) -> None:
    """
    将数据复制到剪贴板。

    Parameters:
        data (str): 要复制到剪贴板的数据。

    Returns:
        None
    """
    # 判断data是否为字符串
    if not isinstance(data, str):
        raise ValueError("Data must be a string.")

    # 使用pyperclip库将数据复制到剪贴板
    pyperclip.copy(data)



import sys, shlex
import subprocess
from datetime import datetime, timedelta
from loguru import logger
from typing import Tuple

def exec_command(command: str, timeout=60) -> Tuple[bool, str]:
    """
    执行shell命令

    Args:
        command (str): shell命令
        timeout (int, optional): 超时时间. Defaults to 60.

    Returns:
        tuple: (返回码, 输出)
    """
    data = ''
    try:
        sub = subprocess.run(
            command,
            check=True,
            shell=True,
            timeout=timeout,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT
        )
    except Exception as e:
        logger.error(f'执行{command}报错了,请查看{e}')
        stats = False
    else:
        stats = True
        data = sub.stdout.decode('utf-8')
    return stats, data


def exec_command_realtime_output(cmd, cwd=None, timeout=None, shell=False) -> list:
    """
    执行shell命令，封装了subprocess的Popen方法，实时输出结果，支持超时停止
    :param cmd: 执行命令
    :param cwd: 更改路径
    :param timeout: 超时时间
    :param shell: 是否通过shell执行
    :return: 执行结果 list
    :raises: Exception 执行超时
    :raises: Exception 执行失败
    """
    if shell:
        cmd_list = cmd
    else:
        cmd_list = shlex.split(cmd)
    end_time = 0
    if timeout:
        end_time = datetime.now() + timedelta(seconds=timeout)
    sub = subprocess.Popen(cmd_list, cwd=cwd,
                           stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
                           shell=shell, bufsize=1)
    # 执行输出
    res = []
    while sub.poll() is None:
        output = sub.stdout.readline().decode('utf-8')
        if output:
            print(output.strip())
            res.append(output.strip())
        if timeout:
            if end_time <= datetime.now():
                print(Exception(f'执行超时:{cmd}'))
                raise sys.exit(1)
    # 执行结果
    if sub.returncode == 0:
        return res
    else:
        print(Exception(f'执行失败:{cmd}'))
        raise sys.exit(1)



from ._base import md5_encrypt, base64_encode, base64_decode
from ._fernet_mima import *
from ._tqdm_base import process_with_progress
from ._clipboard import copy_to_clipboard
from ._emoji_message_toast import success_toast, warning_toast, failure_toast


import yagmail  # pip install yagmail
import pendulum

class Mail:
    def __init__(self, host, user, password, port, smtp_ssl=False):
        self.yag = yagmail.SMTP(
            host=host,
            user=user,
            password=password,
            port=port,
            smtp_ssl=smtp_ssl
        )

    def _log(self, content):
        now_time = pendulum.now().to_time_string()
        print(f'INFO [{now_time}]: {content}')

    def send_msg_mail(self, title: str, msg: str, receivers: list):
        """
        发送纯文本邮件

        :param title: 邮件标题
        :param msg: 邮件正文
        :param receivers: 收件人列表
        """
        for receiver in receivers:
            self._log(f'开始给 {receiver} 发送邮件...')
            try:
                self.yag.send(
                    receiver,
                    title,
                    msg
                )
                self._log('发送成功!')
            except Exception as e:
                print(f"{str(e)}\nError: 发送邮件失败!")

    def send_file_mail(self, title: str, msg: str, file_path: str, receivers: list):
        """
        带有附件文件的邮件

        :param title: 邮件标题
        :param msg: 邮件正文
        :param file_path: 附件文件路径
        :param receivers: 收件人列表
        """
        for receiver in receivers:
            self._log(f'开始给 {receiver} 发送邮件...')
            try:
                self.yag.send(
                    receiver,
                    title,
                    [msg, file_path]  # 添加邮件正文和文件路径
                )
                self._log('发送成功!')
            except Exception as e:
                print(f"{str(e)}\nError: 发送邮件失败!")




import re
import ssl
import socket
import dns.resolver
from cryptography import x509
from cryptography.hazmat.backends import default_backend
import pendulum
from typing import Dict, Any

def _is_valid_domain(domain) -> bool:
    """
    验证域名的合法性,未对含有中文的域名做验证
    """
    # 如果域名中含有中文,返回True
    if re.search(r'[\u4e00-\u9fa5]', domain):
        return True

    # 域名的正则表达式
    domain_pattern = r'^([a-zA-Z0-9]([a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?\.)+[a-zA-Z]{2,}$'
    # 编译正则表达式
    regex = re.compile(domain_pattern)
    # 使用 match 方法匹配域名
    if regex.match(domain):
        return True
    else:
        return False

def _convert_utc_to_eastern_8(utc_time):
    """
    将日期时间字符串转换为东八区时间
    """
    # 将字符串转换为 Pendulum 对象
    utc_time = pendulum.parse(str(utc_time))

    # 将时区从 UTC 转换为东八区（北京时间）
    eastern_time = utc_time.in_timezone("Asia/Shanghai").strftime("%Y-%m-%d %H:%M:%S")

    return eastern_time

def _counting_days(after_time):
    """
    计算两个日期时间之间的天数差异
    """
    # 获取当前时间,时区为东八区
    before_time = pendulum.now('Asia/Shanghai')
    after_time = pendulum.parse(str(after_time))

    # 计算天数差异
    days_diff = after_time.diff(before_time).days
    return days_diff

def get_cert_details(domain: str, dns_server='114.114.114.114') -> Dict[str, Any]:
    """
    获取指定域名的证书信息，并返回证书的过期时间和剩余有效天数。

    参数:
    - domain (str): 要查询证书信息的域名。
    - dns_server (str): 用于解析域名的 DNS 服务器地址，默认为 114.114.114.114。

    返回:
    - Dict[str, Any]: 包含证书信息的字典，包括证书版本、主题、颁发者、解析 IP、证书有效期等信息。
    """
    result = {
        'stats': True,
        'msg': '获取域名证书成功🍺🍺🍺🍺🍺',
        'data': ''
    }
    # 使用正则验证域名的合法性
    if not _is_valid_domain(str(domain)):
        result['stats'] = False
        result['msg'] = '域名不合法'
        return result
    try:
        resolver = dns.resolver.Resolver(configure=False)
        resolver.nameservers = [dns_server]
        try:
            ips = resolver.resolve(domain, 'A')
        except Exception:
            raise Exception('解析域名失败,没有获取解析的ip')

        if len(ips) > 1:
            # 获取解析的所有ip
            ips = [str(ip) for ip in ips]
            result['data'] = dict(resolve_ips=ips)
            raise Exception('解析域名成功,但存在多个ip,请手动验证')
        ip = ips[0].to_text()
        # 建立连接并获取服务器证书
        with socket.create_connection((ip, 443), timeout=10) as sock: # 设置超时时间10s
            with ssl.create_default_context().wrap_socket(sock, server_hostname=domain) as ssock:
                der_cert = ssock.getpeercert(True)

        # 解析证书
        cert = x509.load_der_x509_certificate(der_cert, default_backend())

        # 计算过期剩余天数
        not_valid_before = _convert_utc_to_eastern_8(cert.not_valid_before)
        not_valid_after = _convert_utc_to_eastern_8(cert.not_valid_after)
        remaining_days = _counting_days(not_valid_after)

        result['data'] = {
            'version': cert.version,
            'subject': cert.subject.rfc4514_string(),
            'issuer': cert.issuer.rfc4514_string(),
            'resolve_ip': ip,
            'not_valid_before': not_valid_before,
            'not_valid_after': not_valid_after,
            'expiration_days': remaining_days
        }
    except Exception as e:
        result['stats'] = False
        result['msg'] = str(e)

    return result

# if __name__ == "__main__":
#     details = get_cert_details('www.cjdropshipping.com')
#     for k,v in details['data'].items():
#         print(k,v)


import emoji

def _validate_message_type(func):
    def wrapper(message):
        if not isinstance(message, str):
            print(message)  # 如果message不是字符串类型，则直接输出message
            return None  # 返回None，不执行被装饰的函数
        else:
            return func(message)  # 如果message是字符串类型，则执行被装饰的函数
    return wrapper

@_validate_message_type
def success_toast(message: str):
    """
    输出成功消息，并添加成功的表情符号
    """
    success_emoji = emoji.emojize(':thumbs_up:' * 3, language='alias')
    print(f"{success_emoji} Success: {message}")

@_validate_message_type
def warning_toast(message: str):
    """
    输出警告消息，并添加警告的表情符号
    """
    warning_emoji = emoji.emojize(':warning:' * 3, language='alias')
    print(f"{warning_emoji} Warning: {message}")

@_validate_message_type
def failure_toast(message: str):
    """
    输出失败消息，并添加失败的表情符号
    """
    failure_emoji = emoji.emojize(':thumbs_down:' * 3, language='alias')
    print(f"{failure_emoji} Failure: {message}")


# 测试函数
if __name__ == "__main__":
    # 测试成功消息
    success_toast("Task completed successfully!")

    # 测试警告消息
    warning_toast("Low disk space detected.")

    # 测试失败消息
    failure_toast("Error: Connection timed out.")

    # 测试非字符串消息
    failure_toast(123)  # 直接输出数字 123，不执行函数


from tqdm import tqdm

def process_with_progress(iterable, process_func, desc="处理中...") -> list:
    """
    使用 tqdm 封装的通用函数，用于在处理过程中显示进度条。
    对iterable每一项进行process_func处理，并返回处理后的结果列表。
    Parameters:
        iterable (iterable): 要处理的可迭代对象。
        process_func (callable): 处理每个元素的函数。
        desc (str, optional): 进度条的描述信息，默认为 "处理中..."。

    Returns:
        list: 处理后的结果列表。
    """
    results = []
    with tqdm(iterable, desc=desc, unit="item") as pbar:
        for item in pbar:
            result = process_func(item)
            results.append(result)
    return results

"""
# 使用参考
def square(x):
    import time
    time.sleep(0.1)
    return x * x

# 使用 process_with_progress 函数处理一个列表，对每个元素求平方，并显示进度条
input_list = [ n for n in range(1, 100) ]
processed_results = process_with_progress(input_list, square)
print(processed_results)
"""


from cryptography.fernet import Fernet

def fernet_generate_key():
    """
    生成一个随机的Fernet密钥
    """
    return Fernet.generate_key().decode('utf-8')

def fernet_jiami(input_string: str, key: str):
    """
    使用Fernet库对输入的字符串进行加密
    :param input_string: 要加密的字符串
    :param key: Fernet密钥
    """
    # 检查Fernet密钥是否符合格式
    try:
        key = str(key).encode('utf-8')
        # 创建 Fernet 对象
        cipher_suite = Fernet(key)
    except Exception as e:
        mes = f"Fernet密钥格式错误: {e},请使用Fernet.generate_key()生成密钥"
        raise Exception(mes)

    # 将字符串转换为字节型
    input_bytes = input_string.encode('utf-8')
    # 加密字符串
    encrypted_text = cipher_suite.encrypt(input_bytes).decode('utf-8')

    return encrypted_text

def fernet_jiemi(input_string: str, key: str):
    """
    使用Fernet库对输入的字符串进行解密
    :param input_string: 要解密的字符串
    :param key: Fernet密钥
    """

    # 检查Fernet密钥是否符合格式
    try:
        key = str(key).encode('utf-8')
        # 创建 Fernet 对象
        cipher_suite = Fernet(key)
    except Exception as e:
        mes = f"Fernet密钥格式错误: {e}"
        raise Exception(mes)

    # 将字符串转换为字节型
    input_bytes = input_string.encode('utf-8')

    # 解密字符串
    decrypted_text = cipher_suite.decrypt(input_bytes).decode('utf-8')

    return decrypted_text

if __name__ == '__main__':
    # key = Fernet.generate_key().decode('utf-8')
    # print(key)
    s = 'hello world'
    print(fernet_jiami(s, '5nMwLbESgNf5Jb3E36cjzUY50Sa11bCA37yBguyMLHA='))
    print(fernet_jiemi('gAAAAABmIgozSAnuGcLWDl-nCqPqpUSAvX3CN4gziLlJZAZ8GJ3kihQw6VSjBfAIqj3NdjNHjUk8JlO8ylDzPtbGKf8q002HZQ==', '5nMwLbESgNf5Jb3E36cjzUY50Sa11bCA37yBguyMLHA='))
